' CBX_Handler:
' Revision: v1.2.7 (vb)
' (c) 2016 steje - based on starter code from Leo
'
' This is a script for written for Directory Opus v11.
' See http://www.gpsoft.com.au/redirect.asp?page=scripts for development information.

' =====================================================================================
' =========================   EVENT HANDLER: OnDoubleClick   ==========================
'
' The purpose of this script is to help Opus treat comic book archives with a user
' configurable file extension (commonly .cb7, .cbr, or .cbz) similar to how dedicated
' comic archive readers work, where double-clicking on the archive itself automatically
' starts displaying the comic book images inside the archive - but in this case, using
' Opus' internal picture viewer.
'
' For the script to work, Opus must ALREADY be configured to treat files with these
' extensions as archives (7zip, RAR, and ZIP respectively)...
'
' The script also forces the use of the internal picture viewer when double clicking
' on images inside either an archive (if you've browsed inside it as a folder) or a
' real folder that has an extension which matches one from the configurable list of
' archive file extensions.
'
' There is also an option to configure the list of image file extensions that the
' script will handle inside of archives and archive extension folders...
'
' In addition to this primary behavior, the script also provides an option to specify
' a qualifier key that you can use when double-clicking on the comic archives to
' control if the script should do it's job or not. By default, this option is set
' to "none".
'
' NOTE: the script has logic to handle the fairly common tendency for some comic book
' archives to contain a single child folder at the root of the archive, which in turn
' contains the comic images. If there are MULTIPLE items inside the archive root in
' addition to that single child folder, then it's not clear what the "right" thing
' to do is.
'
' If this turns out to be the case, the script will NOT attempt to figure something
' out and will instead just open the archive as normal - like a folder, at which
' point a variation on Leo's original code for the request that generated this
' script will still let the Opus internal picture viewer open any images you double
' click on once you've found them manually inside the archive.
'
' Either way, this also helps in case you want to have another application than Opus
' handle opening images outside of comic archives, but still like the Opus internal
' picture viewer for simple viewing and scrolling through comic book images...
'
' As a side effect of providing a qualifier key that can DISABLE the scripts main
' purpose of execution - if it's otherwise set to run "always on" and without any
' qualifier key to ENABLE script execution, when using the QualKey_Disable option
' I had to specify an explicit action to run instead. This is available as the
' script configuration option Default_Action. I did this because on my system I
' had various alternate actions that happen when double-clicking on comic archive
' files depending on shift|ctrl|alt qualifier keys. My original intent for the
' QualKey_Disable option was that when you use it, I wanted the normal or default
' UN-qualified double-click action to be run against the file... not my normal
' ctrl+double-click action to run (for example - if "ctrl" is what I want to set
' the QualKey_Disable option in this script to).
'
' I also added environment variable control support to the script... To use this
' to control if the script should run or not, set the Always_On option to False,
' and create a regular toggle button or hotkey in Opus that runs something like:
'
'                        @toggle:if $glob:cbx_handler
'
'                        @ifset:$glob:cbx_handler
'                        @set glob:cbx_handler
'
'                        @ifset:else
'                        @set glob:cbx_handler=true
'
' =====================================================================================

Option Explicit
Dim SCRIPT_DEBUG
SCRIPT_DEBUG = False


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Called by Directory Opus to initialize the script
Function OnInit(ByRef initData)

   DOpus.Output("Initializing...")

   ' Provide basic information about the Script
   initData.min_version = "11.16"
   initData.name = "CBX_Handler"
   initData.desc = "Automatically opens comics archives in either the Opus internal " &_
                   "picture viewer, or the user specified default action configured " &_
                   "in the scripts configuration options in Prefs."
   initData.copyright = "(c) 2016 steje"
   initData.version = "v1.2.7 (vb)"
   initData.default_enable = True

   '////////////////////////////////////////////////
   ' The following options are user configurable via the Configure button under the scripts listing in Prefs
   '////////////////////////////////////////////////
   ' Set Always_On to True in order to enable script execution without the use of environment variables
   initData.config.Always_On = True
   '////////////////////////////////////////////////
   ' Set the Archive_Ext_List to the desired comic archives file extensions you want the script to handle
   initData.config.Archive_Ext_List = "*.(cb7|cbr|cbz|zcb)"
   '////////////////////////////////////////////////
   ' Set DEBUG flag to True in order to enable logging messages to the Opus Output Window
   initData.config.DEBUG = False
   '////////////////////////////////////////////////
   ' Set DEBUG_CLEAR flag to True in order to clear log messages from the Opus Output Window between script runs
   initData.config.DEBUG_CLEAR = False
   '////////////////////////////////////////////////
   ' Set the Default_Action to the default Opus command that should be run if the QualKey_Disable option is used
   initData.config.Default_Action = "Go FROMSEL"
   '////////////////////////////////////////////////
   ' Set Fulscreen_Mode to True in order to force Opus to launch the viewer in fullscreen mode if not normally configured to do so in Prefs
   initData.config.Fulscreen_Mode = True
   '////////////////////////////////////////////////
   ' Set the Image_Ext_List to the desired image file extensions you want the script to handle inside archives
   initData.config.Image_Ext_List = "*.(bmp|jpeg|jpg|png)"
   '////////////////////////////////////////////////
   ' Set QualKey to the qualifier key that you want to ENABLE the scripts execution
   initData.config.QualKey = DOpus.NewVector(0, "none", "shift", "ctrl", "alt", "lwin", "rwin")
   '////////////////////////////////////////////////
   ' Set QualKey_Disable to the qualifier key that you want to DISABLE the scripts execution
   initData.config.QualKey_Disable = DOpus.NewVector(0, "none", "shift", "ctrl", "alt", "lwin", "rwin")
   '////////////////////////////////////////////////
   ' Set descriptions for all script options
   initData.config_desc = DOpus.NewMap("Always_On", "Set this option to True in order to enable script execution without the use of environment variables.", _
                                       "Archive_Ext_List", "This option lets you specify the list of comic archive file extensions you want the script to handle.", _
                                       "DEBUG", "Set this option to True to enable logging to the Opus script log.", _
                                       "DEBUG_CLEAR", "Set this option to True clear messages logged to the Opus script log between script runs.", _
                                       "Default_Action", "Set this option to the default Opus command that should be run if the QualKey_Disable option is used.", _
                                       "Fulscreen_Mode", "Set this option to True to force the Opus viewer to launch in fullscreen mode.", _
                                       "Image_Ext_List", "This option lets you specify the list of image file extensions you want the script to handle inside archives.", _
                                       "QualKey", "Set this option to the qualifier key that you want to ENABLE the scripts execution.", _
                                       "QualKey_Disable", "Set this option to the qualifier key that you want to DISABLE the scripts execution.")

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Called when an item is double-clicked
Function OnDoubleClick(doubleClickData)
   Dim objCmd
   Dim fRunDefault
   Dim archiveTest, filextTest, folderTest
   Const vbQuote = """"

   Set objCmd = DOpus.NewCommand
   If (objCmd.IsSet("$glob:SCRIPT_DEBUG")) Then SCRIPT_DEBUG = True
   If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) And (Script.config.DEBUG_CLEAR) Then DOpus.ClearOutput

   OnDoubleClick = False
   fRunDefault = False

   ' Set wildcard patterns based on script configuration options
   Set archiveTest = DOpus.FSUtil.NewWild(Script.config.Archive_Ext_List)
   Set folderTest = DOpus.FSUtil.NewWild(Script.config.Archive_Ext_List & "*")
   Set filextTest = DOpus.FSUtil.NewWild(Script.config.Image_Ext_List)

   logMsg("")
   logMsg("----------------------------------------------------------------")
   logMsg("OnDoubleClick:  " & doubleClickData.item)
   If (executeScript(doubleClickData, fRunDefault)) Then
      OnDoubleClick = handle_CBX(doubleClickData, archiveTest, filextTest, folderTest)
   ElseIf ((fRunDefault) And (Script.config.Default_Action <> "") And (archiveTest.Match(doubleClickData.item))) Then
      logMsg("Run Default_Action:  " & Script.config.Default_Action)
      OnDoubleClick = Script.config.Default_Action
   End If

   Set archiveTest = Nothing
   Set folderTest = Nothing
   Set filextTest = Nothing
   Set objCmd = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Main worker function of the script for OnDoubleClick events
Function handle_CBX(ByRef doubleClickData, ByRef archiveTest, ByRef filextTest, ByRef folderTest)
   Dim folderEnum, folderItem
   Dim strCommand, showArgs
   Const vbQuote = """"
   'logMsg("handle_CBX()")

   ' Code from steje to open comic images automatically just by double-clicking on the archive
   ' Overrides the normal user configured Opus behavior of opening the archive as a folder...
   handle_CBX = False

   'Get Fullscreen option for Show command
   If (Script.config.Fulscreen_Mode) Then
      showArgs = " fullscreen"
   Else
      showArgs = ""
   End If

   ' Don't override REAL folders...
   If (DOpus.FSUtil.GetType(doubleClickData.item, "a") = "dir") Then
      logMsg("----------------------------------------------------------------")
      logMsg("Double-clicked item doesn't match script specifications, exit...")
      Exit Function
   ' If the item that was double-clicked is NOT a comic archive:
   ElseIf Not (archiveTest.Match(doubleClickData.item)) Then
      ' Check if it is inside a comic archive
      If (folderTest.Match(doubleClickData.tab.path)) Then
         If Not (filextTest.Match(doubleClickData.item.ext)) Then
            ' If it's an unsupported file type inside of a comic archive, do nothing and exit
            logMsg("----------------------------------------------------------------")
            logMsg("Double-clicked item doesn't match script specifications, exit...")
            Exit Function
         Else
            ' Based on original code by Leo Davidson for opening images in the viewer when double-clicked from within an archive
            strCommand = "show listsiblings" & showArgs
            logMsg("----------------------------------------------------------------")
            logMsg("RunCommand:  " & strCommand)
            handle_CBX = strCommand
            Exit Function
         End If
      Else
         logMsg("----------------------------------------------------------------")
         logMsg("Double-clicked item doesn't match script specifications, exit...")
         Exit Function
      End If
   ' If the item that was double-clicked is a comic archive:
   ElseIf (archiveTest.Match(doubleClickData.item)) Then
      ' It's not uncommon for comic archives to have a single child folder in the root of the archive.
      Set folderEnum = DOpus.FSUtil.ReadDir(doubleClickData.item)
      Set folderItem = folderEnum.Next
      ' If the first item in the archive is a directory:
      If (DOpus.FSUtil.GetType(folderItem, "a") = "dir") Then
         ' If there are additional items in the root than just that first child folder, it's not really
         ' clear what the "right" thing to do is, so perform the default
         If Not (folderEnum.complete) Then
            If (Script.config.Default_Action <> "") Then
               logMsg("----------------------------------------------------------------")
               logMsg("Comic Archive has multiple non-images at the root...")
               logMsg("Run Default_Action:  " & Script.config.Default_Action)
               handle_CBX = Script.config.Default_Action
            Else
               logMsg("----------------------------------------------------------------")
               logMsg("Comic Archive has multiple non-images at the root...")
               logMsg("No Default_Action is configured, exit...")
            End If
         ' If there was only the single child folder at the root, then treat it as one of those archives
         ' that happen to come this way and we can try to show images from that that child folder path...
         Else
            strCommand = "show " & vbQuote & folderItem & "\" & filextTest & vbQuote & " listsiblings" & showArgs
            logMsg("----------------------------------------------------------------")
            logMsg("RunCommand:  " & strCommand)
            handle_CBX = strCommand
         End If
      ' If the first item in the archive is NOT a directory go ahead and assume it's a standard comic
      ' archive and try to show images from the archives root path...
      Else
         strCommand = "show " & vbQuote & doubleClickData.item & "\" & filextTest & vbQuote & " listsiblings" & showArgs
         logMsg("----------------------------------------------------------------")
         logMsg("RunCommand:  " & strCommand)
         handle_CBX = strCommand
      End If
      folderEnum.Close

   End If

   Set folderEnum = Nothing
   Set folderItem = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Check for various things that determine if the script should run or not
Function executeScript(ByRef doubleClickData, ByRef fRunDefault)
   Dim QUALKEY
   Dim objCmd
   'logMsg("executeScript()")

   executeScript = False

   ' Check for any of the script options or environment variables that enable the scripts execution
   Set objCmd = DOpus.NewCommand
   If (Script.config.Always_On) Then
      'logMsg("Script is set to 'always on'")
   ElseIf(objCmd.IsSet("$glob:cbx_handler")) Then
      'logMsg("[cbx_handler] is set: [global]")
   ElseIf (objCmd.IsSet("$lst:cbx_handler")) Then
      'logMsg("[cbx_handler] is set: [lister]")
   ElseIf (objCmd.IsSet("$src:cbx_handler")) Then
      'logMsg("[cbx_handler] is set: [source tab]")
   Else
      logMsg("----------------------------------------------------------------")
      logMsg("Not enabled, exit...")
      Set objCmd = Nothing
      Exit Function
   End If

   ' Check for any qualifier key that might have been used and recast as int we can test against vector values
   If (doubleClickData.qualifiers = "none") Then
      QUALKEY = 0
   ElseIf (doubleClickData.qualifiers = "shift") Then
      QUALKEY = 1
   ElseIf (doubleClickData.qualifiers = "ctrl") Then
      QUALKEY = 2
   ElseIf (doubleClickData.qualifiers = "alt") Then
      QUALKEY = 3
   ElseIf (doubleClickData.qualifiers = "lwin") Then
      QUALKEY = 4
   ElseIf (doubleClickData.qualifiers = "rwin") Then
      QUALKEY = 5
   End If

   ' Check for DISABLING qualifier key settings
   If ((QUALKEY = Script.config.QualKey_Disable) And (QUALKEY > 0)) Then
      logMsg("----------------------------------------------------------------")
      logMsg("The qualifier key to DISABLE was used, exit...")
      fRunDefault = True
      Set objCmd = Nothing
      Exit Function
   End If

   ' Check for ENABLING qualifier key settings
   If Not (QUALKEY = Script.config.QualKey) Then
      logMsg("----------------------------------------------------------------")
      logMsg("No valid qualifier key was used, exit...")
      Set objCmd = Nothing
      Exit Function
   End If

   Set objCmd = Nothing
   executeScript = True

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Subroutine that uses DEBUG variables to control whether logging is performed or not
Sub logMsg(ByRef message)

   If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) Then DOpus.Output(message)

End Sub
